@@ -15,11 +15,6 @@ from scipy.interpolate import interp1d
 from p_winds import parker, tools, microphysics
 
 
-import warnings
-warnings.warn('`p-winds` versions >=1.2 will have a change in the syntax of '
-              '`hydrogen.ion_fraction()`.', FutureWarning)
-
-
 __all__ = ["radiative_processes_exact", "radiative_processes",
            "radiative_processes_mono", "recombination", "ion_fraction"]
 
@@ -39,13 +34,13 @@ def radiative_processes_exact(spectrum_at_planet, r_grid, density, f_r,
         least up to the wavelength corresponding to the energy to ionize
         hydrogen (13.6 eV, or 911.65 Angstrom).
     
-    r_grid (``array``):
+    r_grid (``numpy.ndarray``):
         Radius grid for the calculation, in units of cm.
 
-    density (``array``):
+    density (``numpy.ndarray``):
         Number density profile for the atmosphere, in units of 1 / cm ** 3.
 
-    f_r (``array``):
+    f_r (``numpy.ndarray`` or ``float``):
         Ionization fraction profile for the atmosphere.
 
     h_fraction (``float``):
@@ -221,10 +216,11 @@ def recombination(temperature):
 
 # Fraction of ionized hydrogen vs. radius profile
 def ion_fraction(radius_profile, planet_radius, temperature, h_fraction,
-                 mass_loss_rate, planet_mass, average_ion_fraction=0.0,
+                 mass_loss_rate, planet_mass, mean_molecular_weight_0=1.0,
+                 star_mass = 1.0, semimajor_axis = 1.0,
                  spectrum_at_planet=None, flux_euv=None, initial_f_ion=0.0,
                  relax_solution=False, convergence=0.01, max_n_relax=10,
-                 exact_phi=False, **options_solve_ivp):
+                 exact_phi=False, return_mu=False, **options_solve_ivp):
     """
     Calculate the fraction of ionized hydrogen in the upper atmosphere in
     function of the radius in unit of planetary radius.
@@ -249,8 +245,20 @@ def ion_fraction(radius_profile, planet_radius, temperature, h_fraction,
     planet_mass (``float``):
         Planetary mass in unit of Jupiter mass.
 
-    average_ion_fraction (``float``):
-        Average ion fraction in the upper atmosphere.
+    mean_molecular_weight_0 (``float``):
+        Initial mean molecular weight of the atmosphere in unit of proton mass.
+        Default value is 1.0 (100% neutral H). Since its final value depend on
+        the H ion fraction itself, the mean molecular weight can be
+        self-consistently calculated by setting `relax_solution` to `True`.
+
+    star_mass (``float``, optional):
+        Stellar mass in units of M_sun, needed for the tidal gravity
+        calculation. Default is 1.
+
+    semimajor_axis (``float``, optional):
+        Planetary semimajor axis in units of au, needed for the tidal gravity
+        calculation. Default is 1 (so the tidal gravity correction is minimal by
+        default).
 
     spectrum_at_planet (``dict``, optional):
         Spectrum of the host star arriving at the planet covering fluxes at
@@ -282,6 +290,12 @@ def ion_fraction(radius_profile, planet_radius, temperature, h_fraction,
         Maximum number of loops to perform the relaxation of the solution for
         ``f_r``. Default is 10.
 
+    return_mu (``bool``, optional):
+        If ``True``, then this function returns a second variable ``mu_bar``,
+        which is the self-consistent, density-averaged mean molecular weight of
+        the atmosphere. Equivalent to the ``mu_bar`` of Eq. A.3 in Lampón et
+        al. 2020.
+
     **options_solve_ivp:
         Options to be passed to the ``scipy.integrate.solve_ivp()`` solver. You
         may want to change the options ``method`` (integration method; default
@@ -294,6 +308,12 @@ def ion_fraction(radius_profile, planet_radius, temperature, h_fraction,
     -------
     f_r (``numpy.ndarray``):
         Values of the fraction of ionized hydrogen in function of the radius.
+
+    mu_bar (``float``):
+        Mean molecular weight of the atmosphere, in unit of proton mass,
+        averaged across the radial distance using according to the function
+        `average_molecular_weight` in the `parker` module. Only returned when
+        ``return_mu`` is set to ``True``.
     """
     # Hydrogen recombination rate
     alpha_rec = recombination(temperature)
@@ -302,16 +322,20 @@ def ion_fraction(radius_profile, planet_radius, temperature, h_fraction,
     m_h = 1.67262192E-24
 
     # Photoionization rate at null optical depth at the distance of the planet
-    # from the host star, in unit of vs / rs
+    # from the host star, in unit of 1 / s.
     if exact_phi and spectrum_at_planet is not None:
-        vs = parker.sound_speed(temperature, h_fraction, average_ion_fraction)
-        rs = parker.radius_sonic_point(planet_mass, vs)
+        vs = parker.sound_speed(temperature, mean_molecular_weight_0)
+        rs = parker.radius_sonic_point_tidal(planet_mass, vs, star_mass,
+                                             semimajor_axis)
         rhos = parker.density_sonic_point(mass_loss_rate, rs, vs)
-        _, rho_norm = parker.structure(radius_profile * planet_radius / rs)
+        _, rho_norm = parker.structure_tidal(
+            radius_profile * planet_radius / rs, vs, rs, planet_mass,
+            star_mass, semimajor_axis)
+        f_outer = 0.0  # Assume completely ionized at the top of atmosphere
         phi_abs = radiative_processes_exact(
             spectrum_at_planet,
             (radius_profile * planet_radius * u.Rjup).to(u.cm).value,
-            rho_norm * rhos, average_ion_fraction, h_fraction)
+            rho_norm * rhos, f_outer, h_fraction)
         a_0 = 0.
     elif spectrum_at_planet is not None:
         phi_abs, a_0 = radiative_processes(spectrum_at_planet)
@@ -325,21 +349,23 @@ def ion_fraction(radius_profile, planet_radius, temperature, h_fraction,
     # cm ** 2 / g
     # We assume that the remaining of the number fraction is pure He
     he_fraction = 1 - h_fraction
-    k1_abs = (h_fraction * a_0 / (h_fraction + he_fraction * 4) / m_h)
+    he_h_fraction = he_fraction / h_fraction
+    k1_abs = h_fraction * a_0 / (h_fraction + 4 * he_fraction) / m_h
 
     # Multiplicative factor of the second term in the right-hand side of Eq.
     # 13 of Oklopcic & Hirata 2018, unit of cm ** 3 / s / g
-    k2_abs = h_fraction / (h_fraction + he_fraction * 4) * alpha_rec / m_h
+    k2_abs = h_fraction / (h_fraction + 4 * he_fraction) * alpha_rec / m_h
 
     # In order to avoid numerical overflows, we need to normalize a few key
     # variables. Since the normalization may need to be repeated to relax the
     # solution, we have a function to do it.
-    def _normalize(_phi, _k1, _k2, _r, _mean_f_ion):
+    def _normalize(_phi, _k1, _k2, _r, _mu):
         # First calculate the sound speed, radius at the sonic point and the
         # density at the sonic point. They will be useful to change the units of
         # the calculation aiming to avoid numerical overflows
-        _vs = parker.sound_speed(temperature, h_fraction, _mean_f_ion)
-        _rs = parker.radius_sonic_point(planet_mass, _vs)
+        _vs = parker.sound_speed(temperature, _mu)
+        _rs = parker.radius_sonic_point_tidal(planet_mass, _vs, star_mass,
+                                              semimajor_axis)
         _rhos = parker.density_sonic_point(mass_loss_rate, _rs, _vs)
         # And now normalize everything
         phi_unit = _vs * 1E5 / _rs / 7.1492E+09  # 1 / s
@@ -355,12 +381,13 @@ def ion_fraction(radius_profile, planet_radius, temperature, h_fraction,
         dr_norm = np.concatenate((dr_norm, np.array([dr_norm[-1], ])))
 
         # The structure of the atmosphere
-        v_norm, rho_norm = parker.structure(r_norm)
+        v_norm, rho_norm = parker.structure_tidal(r_norm, _vs, _rs, planet_mass,
+                                                  star_mass, semimajor_axis)
 
         return phi_norm, k1_norm, k2_norm, r_norm, dr_norm, v_norm, rho_norm
 
     phi, k1, k2, r, dr, velocity, density = _normalize(
-        phi_abs, k1_abs, k2_abs, radius_profile, average_ion_fraction)
+        phi_abs, k1_abs, k2_abs, radius_profile, mean_molecular_weight_0)
 
     if exact_phi:
         _phi_prime_fun = interp1d(r, phi, fill_value="extrapolate")
@@ -369,23 +396,32 @@ def ion_fraction(radius_profile, planet_radius, temperature, h_fraction,
         # we don't know it yet, because it depends on the ion fraction in the
         # atmosphere, which is what we want to obtain. However, the optical
         # depth depends more strongly on the densities of H than the ion
-        # fraction, so a good approximation is to assume the whole atmosphere is
-        # neutral at first.
+        # fraction, so a good first approximation is to assume the whole
+        # atmosphere is neutral at first.
         column_density = np.flip(np.cumsum(np.flip(dr * density)))
         tau_initial = k1 * column_density
-        # We do a dirty hack to make tau_initial a callable function so it's
-        # easily parsed inside the differential equation solver
+        # We do a dirty hack to make tau_initial and velocity a callable
+        # function so it's easily parsed inside the differential equation solver
         _tau_fun = interp1d(r, tau_initial, fill_value="extrapolate")
+    _v_fun = interp1d(r, velocity, fill_value="extrapolate")
+
+    _v_fun = interp1d(r, velocity, fill_value="extrapolate")
+    _rho_fun = interp1d(r, density, fill_value="extrapolate")
 
     # Now let's solve the differential eq. 13 of Oklopcic & Hirata 2018
     # The differential equation in function of r
-    def _fun(_r, _f, _phi, _k1, _k2):
+    def _fun(_r, _f, _phi, _k2):
         if exact_phi:
             _phi_prime = _phi_prime_fun(np.array([_r, ]))[0]
         else:
             _t = _tau_fun(np.array([_r, ]))[0]
             _phi_prime = np.exp(-_t)*_phi
-        _v, _rho = parker.structure(_r)
+
+        # The next two lines may need to be substituted by `structure_tidal()`
+        # instead of interpolated
+        _v = _v_fun(_r)
+        _rho = _rho_fun(_r)
+
         # In terms 1 and 2 we use the values of k2 and phi from above
         term1 = (1. - _f) / _v * _phi_prime
         term2 = _k2 * _rho * _f ** 2 / _v
@@ -395,31 +431,39 @@ def ion_fraction(radius_profile, planet_radius, temperature, h_fraction,
 
     # We solve it using `scipy.solve_ivp`
     sol = solve_ivp(_fun, (r[0], r[-1],), np.array([initial_f_ion, ]),
-                    t_eval=r, args=(phi, k1, k2), **options_solve_ivp)
+                    t_eval=r, args=(phi, k2,), 
+                    **options_solve_ivp)
     f_r = sol['y'][0]
+
     # When `solve_ivp` has problems, it may return an array with different
     # size than `r`. So we raise an exception if this happens
     if len(f_r) != len(r):
         raise RuntimeError('The solver ``solve_ivp`` failed to obtain a'
                            ' solution.')
 
+    # Calculate the average mean molecular weight using Eq. A.3 from Lampón et
+    # al. 2020
+    mu_bar = parker.average_molecular_weight(f_r, radius_profile, velocity,
+                                             planet_mass, temperature,
+                                             he_h_fraction)
+
     # For the sake of self-consistency, there is the option of repeating the
     # calculation of f_r by updating the optical depth with the new ion
     # fractions.
     if relax_solution is True:
         for i in range(max_n_relax):
-            previous_f_r_outer_layer = np.copy(f_r)[-1]
-            average_ion_fraction = np.mean(np.copy(f_r))
+            previous_f_r = np.copy(f_r)
             
             if exact_phi:
                 # phi_abs will need to be recomputed here with the new density
                 # structure
-                vs = parker.sound_speed(temperature, h_fraction,
-                    average_ion_fraction)
-                rs = parker.radius_sonic_point(planet_mass, vs)
+                vs = parker.sound_speed(temperature, mu_bar)
+                rs = parker.radius_sonic_point_tidal(planet_mass, vs, star_mass,
+                                                     semimajor_axis)
                 rhos = parker.density_sonic_point(mass_loss_rate, rs, vs)
-                _, rho_norm = parker.structure(
-                    radius_profile * planet_radius / rs)
+                _, rho_norm = parker.structure_tidal(
+                    radius_profile * planet_radius / rs, vs, rs, planet_mass,
+                    star_mass, semimajor_axis)
                 phi_abs = radiative_processes_exact(
                     spectrum_at_planet,
                     (radius_profile * planet_radius * u.Rjup).to(u.cm).value,
@@ -428,7 +472,7 @@ def ion_fraction(radius_profile, planet_radius, temperature, h_fraction,
             # We re-normalize key parameters because the newly-calculated f_ion
             # changes the value of the mean molecular weight of the atmosphere
             phi, k1, k2, r, dr, velocity, density = _normalize(
-                phi_abs, k1_abs, k2_abs, radius_profile, average_ion_fraction)
+                phi_abs, k1_abs, k2_abs, radius_profile, mu_bar)
 
             if exact_phi:
                 _phi_prime_fun = interp1d(r, phi, fill_value="extrapolate")
@@ -438,10 +482,12 @@ def ion_fraction(radius_profile, planet_radius, temperature, h_fraction,
                                                            (1 - f_r))))
                 tau = k1 * column_density
                 _tau_fun = interp1d(r, tau, fill_value="extrapolate")
+            _v_fun = interp1d(r, velocity, fill_value="extrapolate")
             
             # And solve it again
             sol = solve_ivp(_fun, (r[0], r[-1],), np.array([initial_f_ion, ]),
-                            t_eval=r, args=(phi, k1, k2), **options_solve_ivp)
+                            t_eval=r, args=(phi, k2,), 
+                            **options_solve_ivp)
             f_r = sol['y'][0]
 
             # Raise an error if the length of `f_r` is different from the length
@@ -450,10 +496,18 @@ def ion_fraction(radius_profile, planet_radius, temperature, h_fraction,
                 raise RuntimeError('The solver ``solve_ivp`` failed to obtain a'
                                    ' solution.')
 
+            # Here we update the average mean molecular weight
+            mu_bar = parker.average_molecular_weight(f_r, radius_profile,
+                                                     velocity,
+                                                     planet_mass, temperature,
+                                                     he_h_fraction)
+
             # Calculate the relative change of f_ion in the outer shell of the
             # atmosphere (where we expect the most important change)
-            relative_delta_f = abs(f_r[-1] - previous_f_r_outer_layer) \
-                / previous_f_r_outer_layer
+            # relative_delta_f = abs(f_r[-1] - previous_f_r_outer_layer) \
+            #     / previous_f_r_outer_layer
+            relative_delta_f = abs(
+                np.sum(f_r - previous_f_r) / np.sum(previous_f_r))
 
             # Break the loop if convergence is achieved
             if relative_delta_f < convergence:
@@ -462,4 +516,8 @@ def ion_fraction(radius_profile, planet_radius, temperature, h_fraction,
                 pass
     else:
         pass
-    return f_r
+
+    if return_mu is False:
+        return f_r
+    else:
+        return f_r, mu_bar
